import typing as T
import random


class TracesException(Exception):
    def __init__(
        self, message: T.Optional[str] = "", traces_rate: T.Optional[float] = 1
    ):
        """
        :param message: the error message to display to the user
        :param traces_rate: the traces rate for Sentry, between 0 and 1, inclusive
        """
        self.message = message
        self.traces_rate = traces_rate
        super().__init__(self.message)

    def should_capture(self) -> bool:
        if self.traces_rate == 1:
            return True
        if self.traces_rate == 0:
            return False
        return random.random() < self.traces_rate


class DisplayException(TracesException):
    @classmethod
    def which_exception(cls, e: Exception) -> Exception:
        str_e = str(e)
        if str_e.startswith("*"):
            return cls(message=str_e[1:])
        return e



import httpx
from pydantic import BaseModel
from quilt.vendors.GooglePlaces.main import Place, Address
from quilt import DisplayException

TZ_FROM_COORDS_URL = (
    "https://getzipcodeinfo.herokuapp.com/get_tz_from_coords?lat=<LAT>&lng=<LNG>"
)


class LocationError(DisplayException):
    pass


def tz_from_coords(lat: float, lng: float) -> str:
    url = TZ_FROM_COORDS_URL.replace("<LAT>", str(lat)).replace("<LNG>", str(lng))
    return httpx.get(url).json()


class Location(BaseModel):
    lat: float
    lng: float
    city: str | None = None
    state_abbr: str | None = None
    country_name: str | None = None
    postal_code: str | None = None
    street_address: str | None = None

    formatted_address: str | None = None

    timezone: str
    place_id: str

    @classmethod
    def from_place(cls, place: Place) -> "Location":
        address = Address.from_place(place)
        loc = place.geometry.location
        return cls(
            lat=loc.lat,
            lng=loc.lng,
            city=address.city,
            state_abbr=address.state_abbr,
            country_name=address.country_name,
            postal_code=address.postal_code,
            street_address=address.street_address,
            formatted_address=place.formatted_address,
            timezone=tz_from_coords(lat=loc.lat, lng=loc.lng),
            place_id=place.place_id,
        )

    @classmethod
    async def from_place_id(
        cls, place_id: str, force_street_address: bool = True
    ) -> "Location":
        place = await Place.get(place_id=place_id)
        loc = cls.from_place(place)
        if force_street_address and not loc.street_address:
            raise LocationError(
                "Street Address required but not given. Please choose a location with a street address."
            )
        return loc


import strawberry


@strawberry.experimental.pydantic.type(model=Location, name="Location", all_fields=True)
class LocationStraw:
    pass


@strawberry.experimental.pydantic.input(
    model=Location, name="LocationInput", all_fields=True
)
class LocationStrawInput:
    pass



import os
import json
import firebase_admin
from firebase_admin import auth
from firebase_admin import credentials


cred = credentials.Certificate(json.loads(os.environ["FIREBASE_CERT"]))
client = firebase_admin.initialize_app(cred)


def update_user(
    auth_id: str,
    display_name: str = None,
    email: str = None,
    email_verified: bool = None,
    # phone_number: bool = None,
    photo_url: str = None,
    disabled: bool = None,
    custom_claims: dict[str, str] = None,
) -> auth.UserRecord:
    kwargs = {
        "display_name": display_name,
        "email": email,
        "photo_url": photo_url,
        "disabled": disabled,
        "custom_claims": custom_claims,
    }
    kwargs = {k: v for k, v in kwargs.items() if v is not None}
    return auth.update_user(uid=auth_id, **kwargs)



import typing as T
import time
from contextlib import contextmanager
import sentry_sdk
from quilt import logs

logger = logs.create_logger(__name__)


@contextmanager
def timer(op: str, description: str = None) -> T.Iterator[None]:
    start = time.perf_counter()
    yield
    end = time.perf_counter()
    took_ms = (end - start) * 1_000
    logger.debug(f"{op=}, {description=}, took {took_ms:.3} ms")


@contextmanager
def span(
    op: str,
    description: str = None,
    use_parent: bool = True,
    use: bool = True,
    log: bool = False,
    **kwargs: T.Any,
) -> T.Iterator[T.Any]:
    if not use:
        if log:
            with timer(op=op, description=description):
                yield
        else:
            yield
    else:
        current_span = None
        if use_parent:
            current_span = sentry_sdk.Hub.current.scope.span
        if current_span:
            context = current_span.start_child(op=op, description=description, **kwargs)
        else:
            context = sentry_sdk.start_span(op=op, description=description, **kwargs)
        with context:
            if log:
                with timer(op=op, description=description):
                    yield
            else:
                yield



import typing as T
import time
from fastapi import Request, Response, FastAPI
from quilt import logs

logger = logs.create_logger(__name__, one_color=logs.Color.PURPLE)

PROCESS_TIME_HEADER_NAME = "X-Process-Time-Ms"


def add_error_handling_and_process_time(app: FastAPI) -> None:
    @app.middleware("http")
    async def add_error_handling_and_process_time_header(
        request: Request, call_next: T.Any
    ) -> Response:
        start_time = time.time()
        response = await call_next(request)
        process_time = time.time() - start_time
        response.headers[PROCESS_TIME_HEADER_NAME] = str(process_time * 1_000)
        logger.debug("\n***Request Done***\n")
        return response



import os
import time
import httpx
import functools
from pydantic import BaseModel


base_url = "https://ipinfo.io"


class IpInfoResponse(BaseModel):
    ip: str
    hostname: str | None = None
    city: str | None = None
    region: str | None = None
    country: str | None = None
    loc: str | None = None
    org: str | None = None
    postal: str | None = None
    timezone: str | None = None
    bogon: bool | None = None

    def get_location(self) -> str:
        if not self.city and not self.region and not self.country:
            return "Local Request"
        return f"{self.city}, {self.region} in {self.country}"


@functools.lru_cache(maxsize=1_000)
def get_ip_info(ip_address: str) -> IpInfoResponse | None:
    if not ip_address:
        return None
    res = httpx.get(f"{base_url}/{ip_address}?token={os.environ['IP_INFO_TOKEN']}")
    if res.status_code == 200:
        return IpInfoResponse(**res.json())
    return None


if __name__ == "__main__":
    start = time.time()
    print(get_ip_info(ip_address="51.91.31.157").get_location())  # type: ignore
    print(time.time() - start)



import typing as T
import inspect
from enum import Enum
from strawberry.type import StrawberryList
from strawberry.union import StrawberryUnion
from strawberry.types.nodes import SelectedField, FragmentSpread, InlineFragment
from quilt.gql_utils.strawberry_utils import StrawMixin
from quilt.gql_utils.resolvers.models import Preload, kwargs_from_selected_field
from edge_orm import Resolver
from app.api import Info
from quilt import helpers, logs
from .helpers import selected_field_from_path, ResolverUtilException

logger = logs.create_logger(__name__)

StrawType = T.TypeVar("StrawType", bound=StrawMixin[T.Any])
ResolverType = T.TypeVar("ResolverType", bound=Resolver)


async def update_rez(
    rez: Resolver,
    straw_type: T.Type[StrawType],
    selected_field: SelectedField,
    auth_id: str | None,
    type_condition: str | None = None,
) -> set[Enum]:
    """this should return a set of all permissions found"""
    permissions: set[Enum] = set()
    selections = selected_field.selections.copy()
    while selections:
        child_field = selections.pop(0)
        if isinstance(child_field, (FragmentSpread, InlineFragment)):
            if (
                type_condition
                and isinstance(child_field, InlineFragment)
                and child_field.type_condition != type_condition
            ):
                continue
            selections.extend(child_field.selections)
            continue
        child_field_name = helpers.camel_to_snake(child_field.name)
        if child_field_name in rez._node_config.node_edgedb_conversion_map:
            rez.include_fields(child_field_name)
        # find the function on the straw_type
        straw_field: T.Callable[..., T.Any] | None = getattr(
            straw_type, child_field_name, None
        )
        if straw_field is None:
            # Non strawberry_field fields; for example, 'id' field. Already included so move on.
            continue
        if preloads := straw_field.__dict__.get("preloads"):
            preloads = T.cast(list[Preload], preloads)
            for preload in preloads:
                if preload.permissions:
                    permissions.update(preload.permissions)
                if not preload.update_resolver_f_given:
                    nested_rez = preload.update_resolver_f(
                        rez=rez,
                        selected_field=child_field,
                        return_type=preload.skip_to.return_type,
                        auth_id=auth_id,
                    )
                else:
                    kwargs = kwargs_from_selected_field(
                        rez_function=preload.update_resolver_f,
                        selected_field=child_field,
                        auth_id=auth_id,
                    )
                    nested_rez = preload.update_resolver_f(rez=rez, **kwargs)
                    if preload.fields_to_include:
                        rez.include_fields(*preload.fields_to_include)
                if inspect.isawaitable(nested_rez):
                    nested_rez = await nested_rez
                if nested_rez:
                    if preload.skip_to.path_to_return_type:
                        try:
                            child_field = selected_field_from_path(
                                top_field=child_field,
                                path=preload.skip_to.path_to_return_type,
                            )
                        except ResolverUtilException:
                            continue
                    nested_permissions = await update_rez(
                        rez=nested_rez,
                        straw_type=preload.skip_to.return_type,
                        selected_field=child_field,
                        auth_id=auth_id,
                    )
                    permissions.update(nested_permissions)
    return permissions


def straw_type_from_union(union: StrawberryUnion) -> T.Type[StrawType]:
    straw_type = None
    for t in union.types:
        if issubclass(t, StrawMixin):
            if straw_type is not None:
                raise Exception(
                    "There is more than one straw type in the union "
                    "so you must specify which one with the `return_straw_type` field."
                )
            straw_type = t
    if not straw_type:
        raise Exception("There is no straw type in the response union.")
    return straw_type


async def rez_from_info(
    info: Info,
    return_straw_type: T.Optional[T.Type[StrawType]] = None,
    path_to_nested_field: T.Optional[list[str]] = None,
    type_condition: str | None = None,
) -> ResolverType:
    return_straw_type = return_straw_type or info.return_type
    if isinstance(return_straw_type, StrawberryList):
        return_straw_type = return_straw_type.of_type  # type: ignore
    return_straw_type = T.cast(T.Type[StrawType], return_straw_type)
    if isinstance(return_straw_type, StrawberryUnion):
        return_straw_type = straw_type_from_union(return_straw_type)
    rez = return_straw_type.Config.resolver_type()
    top_field = info.selected_fields[0]
    if path_to_nested_field:
        top_field = selected_field_from_path(
            top_field=top_field, path=path_to_nested_field
        )
    permissions = await update_rez(
        rez=rez,
        straw_type=return_straw_type,
        selected_field=top_field,
        auth_id=info.context.auth.auth_id,
        type_condition=type_condition,
    )
    permissions.update(info.context.permissions)
    # now turn permissions into has_permissions_str
    if permissions:
        inner_str = [
            f"<{str(p).split('.')[0]}>'{p.value}' in global current_user.permissions"
            for p in permissions
        ]
        rez.has_permission_str_ = " and ".join(inner_str)
        print(f"{rez.has_permission_str_=}")
    return rez



import typing as T
from multiprocessing.pool import ThreadPool
from pydantic import BaseModel
import strawberry
from enum import Enum


class VerticalAlign(str, Enum):
    TOP = "top"
    CENTER = "center"
    BOTTOM = "bottom"


class Image(BaseModel):
    height: int
    width: int
    url: str
    public_cloudinary_id: str
    vertical_align: VerticalAlign | None = None

    @classmethod
    def from_url_or_bytes(cls, url_or_bytes: T.Union[str, bytes]) -> "Image":
        from quilt.vendors import Cloudinary

        cloudinary_image = Cloudinary.upload_image(url_or_bytes)
        return cls(
            height=cloudinary_image.height,
            width=cloudinary_image.width,
            url=cloudinary_image.secure_url,
            public_cloudinary_id=cloudinary_image.public_id,
        )

    @classmethod
    def from_url_or_bytes_lst(
        cls, url_or_bytes_lst: T.List[T.Union[str, bytes]]
    ) -> T.List["Image"]:
        pool = ThreadPool(processes=5)
        results = []
        for i in url_or_bytes_lst:
            results.append(pool.apply_async(cls.from_url_or_bytes, (i,)))
        return [result.get() for result in results]


strawberry.enum(VerticalAlign)


@strawberry.experimental.pydantic.type(model=Image, name="Image", all_fields=True)
class ImageStraw:
    pass


@strawberry.experimental.pydantic.input(model=Image, name="ImageInput", all_fields=True)
class ImageStrawInput:
    pass


class SmallImage(BaseModel):
    url: str
    public_cloudinary_id: T.Optional[str] = None
    height: T.Optional[int] = None
    width: T.Optional[int] = None


@strawberry.experimental.pydantic.type(
    model=SmallImage, name="SmallImage", all_fields=True
)
class SmallImageStraw:
    pass



import typing

import dateutil.parser
from pydantic import BaseModel
from strawberry import UNSET
from .default_filters import (
    NullsOrderingEnum,
    Ordering,
    OrderingStraw,
    AddToResolverMixin,
    AddToResolverMixinOrder,
)
from .generator import python_filter_name_to_cast
from devtools import debug


class FilterException(Exception):
    pass


filter_field_to_edb_operation = {
    "eq": "=",
    "neq": "!=",
    "gte": ">=",
    "gt": ">",
    "lte": "<=",
    "lt": "<",
    "like": "like",
    "ilike": "ilike",
}


def order_to_str(order: AddToResolverMixinOrder | BaseModel, path: str = "") -> str:
    parts: list[str] = []
    # d = order.dict() if isinstance(order, BaseModel) else order.__dict__
    d = order.__dict__

    for field_name, val in d.items():
        if val is not UNSET:
            if isinstance(val, Ordering) or isinstance(val, OrderingStraw):
                empty_val = ""
                if val.nulls == NullsOrderingEnum.BIGGEST:
                    empty_val = " EMPTY LAST"
                parts.append(f"{path}.{field_name} {val.dir.value.lower()}{empty_val}")
            else:
                # this is a nested order
                parts.append(order_to_str(val, path=f"{path}.{field_name}"))
    if not parts:
        return ""
    return " THEN ".join(parts)


VARS = dict[str, str]
PARTS_VARS = tuple[str, VARS]


def scalar_filter_to_str(
    f: AddToResolverMixin | BaseModel, root_field_name: str
) -> PARTS_VARS:
    filter_name = f.__class__.__name__
    value_cast = python_filter_name_to_cast.get(filter_name)
    is_enum = False
    if not value_cast:
        value_cast = filter_name[6:]
        is_enum = True

    # d = f.dict() if isinstance(f, BaseModel) else f.__dict__
    d = f.__dict__

    set_d = {k: v for k, v in d.items() if v is not UNSET}
    parts: list[str] = []
    variables: VARS = {}
    for field_name, val in set_d.items():
        use_val = True
        # val_var_name = (root_field_name + field_name + random_str(8)).replace(".", "")
        val_var_name = (root_field_name + field_name).replace(
            ".", ""
        )  # TODO test if this works
        if value_cast == "std::datetime" and type(val) is str:
            val = dateutil.parser.parse(val)
        if field_name in ["in_", "nin"]:
            if field_name == "in_":
                in_or_nin = "IN"
            elif field_name == "nin":
                in_or_nin = "NOT IN"
            else:
                raise Exception(f"must be in_ or nin, is {field_name=}")
            if not is_enum:
                part = f"{root_field_name} {in_or_nin} <{value_cast}>array_unpack(<array<str>>${val_var_name})"
            else:
                part = f"{root_field_name} {in_or_nin} <{value_cast}>array_unpack(<array<str>>${val_var_name})"
        elif field_name == "exists":
            part = f"{'NOT ' if val is False else ''}EXISTS .{root_field_name}"
            use_val = False
        elif operation := filter_field_to_edb_operation.get(field_name):
            part = f".{root_field_name} {operation} <{value_cast}>${val_var_name}"
        else:
            raise FilterException(f"Unknown filter name, {field_name=}, {filter=}")
        parts.append(part)
        if use_val:
            variables[val_var_name] = val
    return " AND ".join(parts).replace("..", "."), variables


def node_filter_to_str(
    f: AddToResolverMixin | BaseModel, root_field_name: str
) -> PARTS_VARS:
    # d = f.dict() if isinstance(f, BaseModel) else f.__dict__
    d = f.__dict__
    return dict_to_filter_and_vars(d, root_field_name=root_field_name)


def dict_to_filter_and_vars(
    d: dict[str, typing.Any], root_field_name: str
) -> tuple[str, dict[str, typing.Any]]:
    variables: VARS = {}
    or_part: str = ""
    and_part: str = ""
    normal_parts: list[str] = []

    set_d = {k: v for k, v in d.items() if v is not UNSET}
    for field_name, val in set_d.items():
        # print(f"{field_name=}, {val=}")
        # if this is a scalar (enum or default, not nested filter)
        if field_name in ["or_", "and_"]:
            nested_parts: list[str] = []
            for nested_filter in val:
                nested_part, nested_vs = node_filter_to_str(
                    f=nested_filter, root_field_name=root_field_name
                )
                if nested_part:
                    nested_parts.append(nested_part)
                    variables.update(nested_vs)
            if field_name == "or_":
                join_str = "OR"
                part = ", ".join([f"({p})" for p in nested_parts])
                part = f"any({{ {part} }})"
            else:
                # Note: could do all() the same way you do any() above. The reason not to do this
                # is because it might actually be desired for AND the way it is -> if empty, make whole thing empty
                join_str = "AND"
                part = f" {join_str} ".join([f"({p})" for p in nested_parts])
                part = f"({part})"
            if len(set_d) > 1:
                part = f"{join_str} {part}"
            if part:
                if field_name == "or_":
                    or_part = part
                else:
                    and_part = part
        elif field_name == "not_":
            part, vs = node_filter_to_str(f=val, root_field_name=root_field_name)
            if part:
                normal_parts.append(f"NOT ({part})")
                variables.update(vs)
        elif field_name == "exists":
            rf = f".{root_field_name}".replace("..", ".")
            s = f"EXISTS {rf}"
            if val is False:
                s = f"NOT {s}"
            normal_parts.append(s)
        else:
            new_root_field_name = f"{root_field_name}.{field_name}"
            if val.__class__.__name__.startswith("NestedFilter"):
                part, vs = node_filter_to_str(
                    f=val, root_field_name=new_root_field_name
                )
                normal_parts.append(part)
                variables.update(vs)
            else:
                part, vs = scalar_filter_to_str(
                    f=val, root_field_name=new_root_field_name
                )
                normal_parts.append(part)
                variables.update(vs)

    normal_str = " AND ".join(normal_parts)
    final_str = f"{normal_str} {and_part} {or_part}"
    final_str = " ".join(final_str.split())

    return final_str, variables



import typing as T
import strawberry
from quilt.models.phone_number import PhoneNumber as PN

PhoneNumber = strawberry.scalar(
    T.NewType("PhoneNumber", PN),
    description="Phone number.",
    serialize=lambda v: v,
    parse_value=lambda v: PN.validate(v),
)



from quilt.services.auth.logic import bearer_token_from_phone_number


def main() -> bytes:
    bearer = bearer_token_from_phone_number(phone_number="+16666666666")
    return bearer


if __name__ == "__main__":
    print(main())
